#include <stdio.h>
#include <stdlib.h>

#ifdef __cplusplus
#  include <iostream.h>
#  include "Dis.h"
#endif

#include "Bias0.h"
#include "DIS.h"
#include "DIS_macros.h"

// Choose between Addition, Multiplication or Division.

#if 1
#  define OPERATION +
#  define DIS_OPERATION DIS_add
#  define MACRO_OP DIS_ADD
#  define Bias_OP BiasAddII
#endif

#if 0
#  define OPERATION *
#  define DIS_OPERATION DIS_mul
#  define MACRO_OP DIS_MUL
#  define Bias_OP BiasMulII
#endif

#if 0
#  define OPERATION /
#  define DIS_OPERATION DIS_div
#  define MACRO_OP DIS_DIV
#  define Bias_OP BiasDivII
#endif


void bench_dis(void);
void bench_cxx(void);
void bench_Cxx(void);
void bench_fp(void);
void bench_macro(void);
void bench_complex(void);
void bench_BIAS(void);
const unsigned int loops = 100000000;
#define UNROLLED_LOOP 1


int main (int argc, char **argv)
{
#if 0
  Dis a = .1;
  int i;
  for (i=0; i<10; i++) {
    a = a * a;
    cout << a << a.dis.s+a.dis.i << endl;
  }
#endif

  if (argc != 2) {
    printf("Bad argument.  Syntax: ./a.out [f|d|c|C|m|x|b].\n");
    return -1;
  }

  switch (argv[1][0]) {
    case 'd': bench_dis(); break;
    case 'c': bench_cxx(); break;
    case 'C': bench_Cxx(); break;
    case 'f': bench_fp(); break;
    case 'm': bench_macro(); break;
    case 'x': bench_complex(); break;
    case 'b': bench_BIAS(); break;
  }
  return 0;
}


void bench_BIAS(void)
{
  int i;
//  unsigned int j;
  BIASINTERVAL b, c;

  c.inf = 1.2; c.sup = 1.3;
  b.inf = 1.0; b.sup = 1.0;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
    Bias_OP( &c, &c, &b);

  printf("c=(%f, %f)\n", c.inf, c.sup);
}

void bench_complex(void)
{
#ifdef __GNUC__
  int i;
//  unsigned int j;
  __complex__ double b, c;

  FPU_set_rounding_to_infinity();
  c = 1.2 - 1.3i;
  b = 1.0 + 1.0i;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
      c = c OPERATION b;

  printf("c=(%f, %f)\n", __real__ c, - __imag__ c);
#endif
}

void bench_macro(void)
{
  int i;
//  unsigned int j;
  DIS b, c;

  FPU_set_rounding_to_infinity();
  c.i = -1.2;
  c.s = 1.2;
  b.i = -1;
  b.s = 1;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
      MACRO_OP(c,c,b);

  printf("c=[%f;%f]\n", -c.i, c.s);
}

void bench_dis(void)
{
  int i;
//  unsigned int j;
  DIS b, c;

  FPU_set_rounding_to_infinity();
  c.i = -1.2;
  c.s = 1.2;
  b.i = -1;
  b.s = 1;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
      c = DIS_OPERATION(c,b);
//      { c.i += b.i ; c.s += b.s; }

  printf("c=[%f;%f]\n", -c.i, c.s);
}

void bench_cxx(void)
{
#ifdef __cplusplus
  int i;
//  unsigned int j;
  Dis_advanced b, c;

  FPU_set_rounding_to_infinity();

  c = 1.2;
  b = 1.0;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
      c = c OPERATION b;
//      c += b;

  FPU_set_rounding_to_nearest();
  cout << c << endl;

#endif
}

void bench_Cxx(void)
{
#ifdef __cplusplus
  int i;
//  unsigned int j;
  Dis b, c;

  c = 1.2;
  b = 1.0;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
      c = c OPERATION b;

  cout << c << endl;
#endif
}

void bench_fp(void)
{
  int i;
//  unsigned int j;
  double b, c;

  b = 1;
  c = 1.2;
  
  for (i=loops/UNROLLED_LOOP; i>0; i--)
//    for (j=0; j<UNROLLED_LOOP; j++)
      c = c OPERATION b;

  printf("c=%f\n", c);
}
